#!/usr/bin/env python3


# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Lunar/solar eclipse tables from 2014 to 2099 - courtesy of Fred Espenak and Jean Meeus.


import datetime


# Eclipse Types.
ECLIPSE_TYPE_ANNULAR = "A"
ECLIPSE_TYPE_HYBRID = "H"
ECLIPSE_TYPE_PARTIAL = "P"
ECLIPSE_TYPE_PENUMBRAL = "N"
ECLIPSE_TYPE_TOTAL = "T"

# Gets the upcoming eclipse, lunar or solar.
#
# dateTimeUTC - Current date/time in UTC as a python datetime.datetime object. 
# isLunar - If True, finds the next lunar eclipse; otherwise finds the next solar eclipse.
#
# Returns a tuple of strings describing the next eclipse:
#    dateTime (format of YYYY-MM-DD HH:MM:SS)
#    eclipseType ("A" = Annular, "H" = Hybrid, "N" = Penumbral, "P" = Partial, "T" = Total)
#    latitude (south is negative)
#    longitude (east is negative).
def getEclipseForUTC( dateTimeUTC, isLunar ):
    if isLunar:
        eclipseData = _lunarEclipseData
    else:
        eclipseData = _solarEclipseData

    eclipseInfo = None
    for eclipse in eclipseData:
        dateTime = datetime.datetime.strptime( eclipse[ 0 ] + ", " + eclipse[ 1 ] + ", " + eclipse[ 2 ] + ", " + eclipse[ 3 ], "%Y, %m, %d, %H:%M:%S" )
        dateTime = dateTime - datetime.timedelta( seconds = int( eclipse[ 4 ] ) ) # Need to subtract delta T (http://eclipse.gsfc.nasa.gov/LEcat5/deltat.html).
        if dateTimeUTC <= dateTime:
            latitude = eclipse[ 6 ][ 0 : len( eclipse[ 6 ] ) - 1 ]
            if eclipse[ 6 ][ -1 ] == "S":
                latitude = "-" + latitude

            longitude = eclipse[ 7 ][ 0 : len( eclipse[ 7 ] ) - 1 ]
            if eclipse[ 7 ][ -1 ] == "E":
                longitude = "-" + longitude

            eclipseInfo = ( str( dateTime ), eclipse[ 5 ], latitude, longitude )
            break

    return eclipseInfo


# http://eclipse.gsfc.nasa.gov/5MCLE/5MKLEcatalog.txt
#
# Title: Five Millennium Catalog of Lunar Eclipses: -1999 to +3000 (2000 BCE to 3000CE)
# Authors: Fred Espenak and Jean Meeus
# Source: NASA Technical Publication NASA/TP-2009-214173
# On Web: http://eclipse.gsfc.nasa.gov/SEpubs/5MKLE.html
# Catalog Key: http://eclipse.gsfc.nasa.gov/LEcat5/LEcatkey.html
# Date: 2011 May 23
# 
#      Year   Month  Day   HH:MM:SS    DT   Type  Lat   Long
_lunarEclipseData = [
    [ "2018", "01", "31", "13:31:00", "71", "T", "17N", "161E" ],
    [ "2018", "07", "27", "20:22:54", "71", "T", "19S", "56E" ],
    [ "2019", "01", "21", "05:13:27", "71", "T", "20N", "75W" ],
    [ "2019", "07", "16", "21:31:55", "71", "P", "22S", "39E" ],
    [ "2020", "01", "10", "19:11:11", "72", "N", "23N", "74E" ],
    [ "2020", "06", "05", "19:26:14", "72", "N", "21S", "69E" ],
    [ "2020", "07", "05", "04:31:12", "72", "N", "24S", "66W" ],
    [ "2020", "11", "30", "09:44:01", "72", "N", "21N", "148W" ],
    [ "2021", "05", "26", "11:19:53", "72", "T", "21S", "170W" ],
    [ "2021", "11", "19", "09:04:06", "73", "P", "19N", "139W" ],
    [ "2022", "05", "16", "04:12:42", "73", "T", "19S", "64W" ],
    [ "2022", "11", "08", "11:00:22", "73", "T", "17N", "169W" ],
    [ "2023", "05", "05", "17:24:05", "73", "N", "17S", "98E" ],
    [ "2023", "10", "28", "20:15:18", "74", "P", "14N", "52E" ],
    [ "2024", "03", "25", "07:13:59", "74", "N", "1S", "106W" ],
    [ "2024", "09", "18", "02:45:25", "74", "P", "3S", "42W" ],
    [ "2025", "03", "14", "06:59:56", "75", "T", "3N", "102W" ],
    [ "2025", "09", "07", "18:12:58", "75", "T", "6S", "87E" ],
    [ "2026", "03", "03", "11:34:52", "75", "T", "6N", "171W" ],
    [ "2026", "08", "28", "04:14:04", "75", "P", "9S", "63W" ],
    [ "2027", "02", "20", "23:14:06", "76", "N", "10N", "15E" ],
    [ "2027", "07", "18", "16:04:09", "76", "N", "22S", "121E" ],
    [ "2027", "08", "17", "07:14:59", "76", "N", "12S", "108W" ],
    [ "2028", "01", "12", "04:14:13", "76", "P", "23N", "61W" ],
    [ "2028", "07", "06", "18:20:57", "77", "P", "23S", "86E" ],
    [ "2028", "12", "31", "16:53:15", "77", "T", "23N", "108E" ],
    [ "2029", "06", "26", "03:23:22", "77", "T", "23S", "50W" ],
    [ "2029", "12", "20", "22:43:12", "78", "T", "23N", "19E" ],
    [ "2030", "06", "15", "18:34:34", "78", "P", "23S", "82E" ],
    [ "2030", "12", "09", "22:28:51", "78", "N", "22N", "21E" ],
    [ "2031", "05", "07", "03:52:02", "78", "N", "18S", "59W" ],
    [ "2031", "06", "05", "11:45:17", "78", "N", "21S", "176W" ],
    [ "2031", "10", "30", "07:46:45", "79", "N", "15N", "121W" ],
    [ "2032", "04", "25", "15:14:51", "79", "T", "14S", "131E" ],
    [ "2032", "10", "18", "19:03:40", "79", "T", "10N", "71E" ],
    [ "2033", "04", "14", "19:13:51", "80", "T", "9S", "72E" ],
    [ "2033", "10", "08", "10:56:23", "80", "T", "6N", "167W" ],
    [ "2034", "04", "03", "19:06:59", "80", "N", "5S", "75E" ],
    [ "2034", "09", "28", "02:47:37", "81", "P", "1N", "44W" ],
    [ "2035", "02", "22", "09:06:12", "81", "N", "9N", "133W" ],
    [ "2035", "08", "19", "01:12:15", "81", "P", "12S", "17W" ],
    [ "2036", "02", "11", "22:13:06", "82", "T", "14N", "31E" ],
    [ "2036", "08", "07", "02:52:32", "82", "T", "16S", "41W" ],
    [ "2037", "01", "31", "14:01:38", "82", "T", "18N", "153E" ],
    [ "2037", "07", "27", "04:09:53", "83", "P", "20S", "60W" ],
    [ "2038", "01", "21", "03:49:52", "83", "N", "21N", "54W" ],
    [ "2038", "06", "17", "02:45:02", "83", "N", "22S", "41W" ],
    [ "2038", "07", "16", "11:35:56", "84", "N", "23S", "172W" ],
    [ "2038", "12", "11", "17:45:00", "84", "N", "22N", "93E" ],
    [ "2039", "06", "06", "18:54:25", "84", "P", "22S", "77E" ],
    [ "2039", "11", "30", "16:56:28", "85", "P", "21N", "104E" ],
    [ "2040", "05", "26", "11:46:22", "85", "T", "21S", "177W" ],
    [ "2040", "11", "18", "19:04:40", "85", "T", "20N", "70E" ],
    [ "2041", "05", "16", "00:43:03", "86", "P", "20S", "12W" ],
    [ "2041", "11", "08", "04:35:05", "86", "P", "18N", "73W" ],
    [ "2042", "04", "05", "14:30:11", "86", "N", "5S", "144E" ],
    [ "2042", "09", "29", "10:45:47", "87", "N", "2N", "163W" ],
    [ "2043", "03", "25", "14:32:04", "87", "T", "2S", "144E" ],
    [ "2043", "09", "19", "01:51:50", "88", "T", "2S", "29W" ],
    [ "2044", "03", "13", "19:38:33", "88", "T", "2N", "68E" ],
    [ "2044", "09", "07", "11:20:44", "88", "T", "5S", "171W" ],
    [ "2045", "03", "03", "07:43:26", "89", "N", "6N", "113W" ],
    [ "2045", "08", "27", "13:54:50", "89", "N", "9S", "152E" ],
    [ "2046", "01", "22", "13:02:37", "90", "P", "21N", "168E" ],
    [ "2046", "07", "18", "01:06:05", "90", "P", "22S", "14W" ],
    [ "2047", "01", "12", "01:26:14", "90", "T", "22N", "19W" ],
    [ "2047", "07", "07", "10:35:45", "91", "T", "23S", "157W" ],
    [ "2048", "01", "01", "06:53:55", "91", "T", "23N", "102W" ],
    [ "2048", "06", "26", "02:02:28", "92", "P", "23S", "30W" ],
    [ "2048", "12", "20", "06:27:48", "92", "N", "22N", "97W" ],
    [ "2049", "05", "17", "11:26:39", "92", "N", "21S", "172W" ],
    [ "2049", "06", "15", "19:14:12", "92", "N", "22S", "72E" ],
    [ "2049", "11", "09", "15:52:11", "93", "N", "18N", "118E" ],
    [ "2050", "05", "06", "22:32:02", "94", "T", "17S", "21E" ],
    [ "2050", "10", "30", "03:21:47", "95", "T", "14N", "54W" ],
    [ "2051", "04", "26", "02:16:28", "96", "T", "13S", "34W" ],
    [ "2051", "10", "19", "19:11:50", "97", "T", "10N", "69E" ],
    [ "2052", "04", "14", "02:18:06", "98", "N", "9S", "34W" ],
    [ "2052", "10", "08", "10:45:58", "99", "P", "5N", "164W" ],
    [ "2053", "03", "04", "17:22:10", "99", "N", "5N", "102E" ],
    [ "2053", "08", "29", "08:05:50", "100", "N", "8S", "121W" ],
    [ "2054", "02", "22", "06:51:27", "101", "T", "10N", "99W" ],
    [ "2054", "08", "18", "09:26:30", "102", "T", "13S", "140W" ],
    [ "2055", "02", "11", "22:46:17", "103", "T", "14N", "22E" ],
    [ "2055", "08", "07", "10:53:18", "104", "P", "17S", "161W" ],
    [ "2056", "02", "01", "12:26:06", "105", "N", "18N", "177E" ],
    [ "2056", "06", "27", "10:03:09", "106", "N", "22S", "149W" ],
    [ "2056", "07", "26", "18:43:24", "106", "N", "20S", "81E" ],
    [ "2056", "12", "22", "01:48:56", "107", "N", "22N", "27W" ],
    [ "2057", "06", "17", "02:26:20", "108", "P", "23S", "36W" ],
    [ "2057", "12", "11", "00:53:38", "109", "P", "23N", "15W" ],
    [ "2058", "06", "06", "19:15:48", "110", "T", "23S", "71E" ],
    [ "2058", "11", "30", "03:16:18", "111", "T", "22N", "52W" ],
    [ "2059", "05", "27", "07:55:35", "112", "P", "22S", "119W" ],
    [ "2059", "11", "19", "13:01:36", "113", "P", "20N", "161E" ],
    [ "2060", "04", "15", "21:37:04", "114", "N", "9S", "37E" ],
    [ "2060", "10", "09", "18:53:32", "115", "N", "6N", "74E" ],
    [ "2060", "11", "08", "04:04:15", "115", "N", "18N", "65W" ],
    [ "2061", "04", "04", "21:54:05", "116", "T", "6S", "33E" ],
    [ "2061", "09", "29", "09:38:13", "117", "T", "2N", "146W" ],
    [ "2062", "03", "25", "03:33:50", "118", "T", "2S", "52W" ],
    [ "2062", "09", "18", "18:34:02", "119", "T", "1S", "80E" ],
    [ "2063", "03", "14", "16:05:49", "120", "P", "1N", "121E" ],
    [ "2063", "09", "07", "20:41:12", "121", "N", "5S", "49E" ],
    [ "2064", "02", "02", "21:48:57", "122", "P", "18N", "37E" ],
    [ "2064", "07", "28", "07:52:48", "123", "P", "20S", "116W" ],
    [ "2065", "01", "22", "09:58:58", "124", "T", "20N", "146W" ],
    [ "2065", "07", "17", "17:48:40", "125", "T", "21S", "95E" ],
    [ "2066", "01", "11", "15:04:47", "126", "T", "21N", "136E" ],
    [ "2066", "07", "07", "09:30:29", "127", "P", "22S", "141W" ],
    [ "2066", "12", "31", "14:30:10", "128", "N", "22N", "144E" ],
    [ "2067", "05", "28", "18:56:08", "129", "N", "23S", "76E" ],
    [ "2067", "06", "27", "02:41:06", "129", "N", "22S", "39W" ],
    [ "2067", "11", "21", "00:04:42", "130", "N", "21N", "4W" ],
    [ "2068", "05", "17", "05:42:17", "131", "P", "20S", "86W" ],
    [ "2068", "11", "09", "11:47:00", "132", "T", "18N", "180E" ],
    [ "2069", "05", "06", "09:09:57", "133", "T", "17S", "138W" ],
    [ "2069", "10", "30", "03:35:06", "134", "T", "14N", "57W" ],
    [ "2070", "04", "25", "09:21:24", "135", "N", "12S", "140W" ],
    [ "2070", "10", "19", "18:51:12", "137", "P", "9N", "74E" ],
    [ "2071", "03", "16", "01:31:09", "137", "N", "1N", "20W" ],
    [ "2071", "09", "09", "15:05:41", "138", "N", "4S", "133E" ],
    [ "2072", "03", "04", "15:23:07", "140", "T", "6N", "133E" ],
    [ "2072", "08", "28", "16:05:42", "141", "T", "9S", "119E" ],
    [ "2073", "02", "22", "07:24:53", "142", "T", "10N", "107W" ],
    [ "2073", "08", "17", "17:42:41", "143", "T", "13S", "96E" ],
    [ "2074", "02", "11", "20:55:58", "144", "N", "15N", "50E" ],
    [ "2074", "07", "08", "17:21:38", "145", "N", "21S", "101E" ],
    [ "2074", "08", "07", "01:56:03", "145", "N", "17S", "27W" ],
    [ "2075", "01", "02", "09:55:03", "146", "N", "22N", "147W" ],
    [ "2075", "06", "28", "09:55:35", "147", "P", "23S", "147W" ],
    [ "2075", "12", "22", "08:55:55", "148", "P", "23N", "134W" ],
    [ "2076", "06", "17", "02:39:47", "149", "T", "23S", "39W" ],
    [ "2076", "12", "10", "11:34:51", "150", "T", "23N", "175W" ],
    [ "2077", "06", "06", "14:59:52", "151", "P", "24S", "135E" ],
    [ "2077", "11", "29", "21:35:53", "152", "P", "23N", "34E" ],
    [ "2078", "04", "27", "04:35:44", "153", "N", "13S", "68W" ],
    [ "2078", "10", "21", "03:08:03", "154", "N", "10N", "50W" ],
    [ "2078", "11", "19", "12:40:04", "154", "N", "21N", "166E" ],
    [ "2079", "04", "16", "05:10:45", "155", "P", "10S", "77W" ],
    [ "2079", "10", "10", "17:30:30", "156", "T", "7N", "95E" ],
    [ "2080", "04", "04", "11:23:38", "157", "T", "6S", "170W" ],
    [ "2080", "09", "29", "01:52:42", "158", "T", "3N", "30W" ],
    [ "2081", "03", "25", "00:22:01", "159", "P", "3S", "4W" ],
    [ "2081", "09", "18", "03:35:26", "161", "N", "1S", "55W" ],
    [ "2082", "02", "13", "06:29:19", "161", "P", "14N", "93W" ],
    [ "2082", "08", "08", "14:46:42", "163", "N", "17S", "141E" ],
    [ "2083", "02", "02", "18:26:46", "164", "T", "17N", "88E" ],
    [ "2083", "07", "29", "01:05:34", "165", "T", "19S", "14W" ],
    [ "2084", "01", "22", "23:13:00", "166", "T", "19N", "15E" ],
    [ "2084", "07", "17", "16:58:51", "167", "P", "20S", "107E" ],
    [ "2085", "01", "10", "22:32:29", "168", "N", "21N", "24E" ],
    [ "2085", "06", "08", "02:17:36", "169", "N", "24S", "34W" ],
    [ "2085", "07", "07", "10:04:40", "169", "N", "21S", "149W" ],
    [ "2085", "12", "01", "08:25:35", "170", "N", "23N", "128W" ],
    [ "2086", "05", "28", "12:43:47", "171", "P", "22S", "169E" ],
    [ "2086", "11", "20", "20:19:42", "172", "P", "20N", "52E" ],
    [ "2087", "05", "17", "15:55:20", "173", "T", "19S", "121E" ],
    [ "2087", "11", "10", "12:05:33", "174", "T", "17N", "175E" ],
    [ "2088", "05", "05", "16:16:50", "175", "P", "16S", "116E" ],
    [ "2088", "10", "30", "03:03:20", "177", "P", "13N", "49W" ],
    [ "2089", "03", "26", "09:34:14", "178", "N", "4S", "142W" ],
    [ "2089", "09", "19", "22:11:17", "179", "N", "0N", "26E" ],
    [ "2090", "03", "15", "23:48:31", "180", "T", "1N", "6E" ],
    [ "2090", "09", "08", "22:52:29", "181", "T", "5S", "17E" ],
    [ "2091", "03", "05", "15:58:22", "182", "T", "6N", "124E" ],
    [ "2091", "08", "29", "00:38:25", "183", "T", "10S", "8W" ],
    [ "2092", "02", "23", "05:20:59", "184", "N", "11N", "76W" ],
    [ "2092", "07", "19", "00:41:58", "185", "N", "19S", "8W" ],
    [ "2092", "08", "17", "09:13:59", "185", "N", "14S", "136W" ],
    [ "2093", "01", "12", "18:00:03", "186", "N", "20N", "93E" ],
    [ "2093", "07", "08", "17:24:18", "187", "P", "22S", "101E" ],
    [ "2094", "01", "01", "17:00:06", "188", "P", "22N", "107E" ],
    [ "2094", "06", "28", "10:01:57", "190", "T", "23S", "149W" ],
    [ "2094", "12", "21", "19:56:32", "191", "T", "24N", "61E" ],
    [ "2095", "06", "17", "22:00:11", "192", "P", "24S", "31E" ],
    [ "2095", "12", "11", "06:15:02", "193", "P", "24N", "95W" ],
    [ "2096", "05", "07", "11:24:42", "194", "N", "16S", "171W" ],
    [ "2096", "06", "06", "02:43:41", "194", "N", "24S", "41W" ],
    [ "2096", "10", "31", "11:30:23", "195", "N", "13N", "175W" ],
    [ "2096", "11", "29", "21:22:22", "195", "N", "23N", "37E" ],
    [ "2097", "04", "26", "12:18:17", "196", "P", "13S", "176E" ],
    [ "2097", "10", "21", "01:30:55", "197", "T", "11N", "26W" ],
    [ "2098", "04", "15", "19:04:48", "198", "T", "10S", "74E" ],
    [ "2098", "10", "10", "09:19:58", "200", "T", "7N", "143W" ],
    [ "2099", "04", "05", "08:30:56", "201", "P", "7S", "127W" ],
    [ "2099", "09", "29", "10:36:38", "202", "N", "3N", "161W" ] ]


# http://eclipse.gsfc.nasa.gov/SEpubs/5MKSE.html
# http://eclipse.gsfc.nasa.gov/5MCSE/5MKSEcatalog.txt
#
# Title: Five Millennium Catalog of Solar Eclipses: -1999 to +3000 (2000 BCE to 3000CE)
# Authors: Fred Espenak and Jean Meeus
# Source: NASA Technical Publication NASA/TP-2008-214170
# Catalog Key: http://sunearth.gsfc.nasa.gov/eclipse/SEcat5/catkey.html
# Date: 2008 10 07
#
#      Year   Month  Day   HH:MM:SS    DT   Type  Lat      Long
_solarEclipseData = [
    [ "2018", "02", "15", "20:52:33", "71", "P", "71.0S", "0.6E" ],
    [ "2018", "07", "13", "03:02:16", "71", "P", "67.9S", "127.4E" ],
    [ "2018", "08", "11", "09:47:28", "71", "P", "70.4N", "174.5E" ],
    [ "2019", "01", "06", "01:42:38", "71", "P", "67.4N", "153.6E" ],
    [ "2019", "07", "02", "19:24:07", "71", "T", "17.4S", "109.0W" ],
    [ "2019", "12", "26", "05:18:53", "72", "A", "1.0N", "102.3E" ],
    [ "2020", "06", "21", "06:41:15", "72", "A", "30.5N", "79.7E" ],
    [ "2020", "12", "14", "16:14:39", "72", "T", "40.3S", "67.9W" ],
    [ "2021", "06", "10", "10:43:07", "72", "A", "80.8N", "66.8W" ],
    [ "2021", "12", "04", "07:34:38", "73", "T", "76.8S", "46.2W" ],
    [ "2022", "04", "30", "20:42:36", "73", "P", "62.1S", "71.5W" ],
    [ "2022", "10", "25", "11:01:20", "73", "P", "61.6N", "77.4E" ],
    [ "2023", "04", "20", "04:17:56", "73", "H", "9.6S", "125.8E" ],
    [ "2023", "10", "14", "18:00:41", "74", "A", "11.4N", "83.1W" ],
    [ "2024", "04", "08", "18:18:29", "74", "T", "25.3N", "104.1W" ],
    [ "2024", "10", "02", "18:46:13", "74", "A", "22.0S", "114.5W" ],
    [ "2025", "03", "29", "10:48:36", "75", "P", "61.1N", "77.1W" ],
    [ "2025", "09", "21", "19:43:04", "75", "P", "60.9S", "153.5E" ],
    [ "2026", "02", "17", "12:13:06", "75", "A", "64.7S", "86.8E" ],
    [ "2026", "08", "12", "17:47:06", "75", "T", "65.2N", "25.2W" ],
    [ "2027", "02", "06", "16:00:48", "76", "A", "31.3S", "48.5W" ],
    [ "2027", "08", "02", "10:07:50", "76", "T", "25.5N", "33.2E" ],
    [ "2028", "01", "26", "15:08:59", "76", "A", "3.0N", "51.5W" ],
    [ "2028", "07", "22", "02:56:40", "77", "T", "15.6S", "126.7E" ],
    [ "2029", "01", "14", "17:13:48", "77", "P", "63.7N", "114.2W" ],
    [ "2029", "06", "12", "04:06:13", "77", "P", "66.8N", "66.2W" ],
    [ "2029", "07", "11", "15:37:19", "77", "P", "64.3S", "85.6W" ],
    [ "2029", "12", "05", "15:03:58", "77", "P", "67.5S", "135.7E" ],
    [ "2030", "06", "01", "06:29:13", "78", "A", "56.5N", "80.1E" ],
    [ "2030", "11", "25", "06:51:37", "78", "T", "43.6S", "71.2E" ],
    [ "2031", "05", "21", "07:16:04", "78", "A", "8.9N", "71.7E" ],
    [ "2031", "11", "14", "21:07:31", "79", "H", "0.6S", "137.6W" ],
    [ "2032", "05", "09", "13:26:42", "79", "A", "51.3S", "7.1W" ],
    [ "2032", "11", "03", "05:34:13", "79", "P", "70.4N", "132.6E" ],
    [ "2033", "03", "30", "18:02:36", "80", "T", "71.3N", "155.8W" ],
    [ "2033", "09", "23", "13:54:31", "80", "P", "72.2S", "121.2W" ],
    [ "2034", "03", "20", "10:18:45", "80", "T", "16.1N", "22.2E" ],
    [ "2034", "09", "12", "16:19:28", "81", "A", "18.2S", "72.6W" ],
    [ "2035", "03", "09", "23:05:54", "81", "A", "29.0S", "154.9W" ],
    [ "2035", "09", "02", "01:56:46", "81", "T", "29.1N", "158.0E" ],
    [ "2036", "02", "27", "04:46:49", "82", "P", "71.6S", "131.4W" ],
    [ "2036", "07", "23", "10:32:06", "82", "P", "68.9S", "3.6E" ],
    [ "2036", "08", "21", "17:25:45", "82", "P", "71.1N", "47.0E" ],
    [ "2037", "01", "16", "09:48:55", "82", "P", "68.5N", "20.8E" ],
    [ "2037", "07", "13", "02:40:36", "83", "T", "24.8S", "139.1E" ],
    [ "2038", "01", "05", "13:47:11", "83", "A", "2.1N", "25.4W" ],
    [ "2038", "07", "02", "13:32:55", "84", "A", "25.4N", "21.9W" ],
    [ "2038", "12", "26", "01:00:10", "84", "T", "40.3S", "164.0E" ],
    [ "2039", "06", "21", "17:12:54", "84", "A", "78.9N", "102.1W" ],
    [ "2039", "12", "15", "16:23:46", "85", "T", "80.9S", "172.8E" ],
    [ "2040", "05", "11", "03:43:02", "85", "P", "62.8S", "174.4E" ],
    [ "2040", "11", "04", "19:09:02", "85", "P", "62.2N", "53.4W" ],
    [ "2041", "04", "30", "11:52:21", "86", "T", "9.6S", "12.2E" ],
    [ "2041", "10", "25", "01:36:22", "86", "A", "9.9N", "162.9E" ],
    [ "2042", "04", "20", "02:17:30", "86", "T", "27.0N", "137.3E" ],
    [ "2042", "10", "14", "02:00:42", "87", "A", "23.7S", "137.8E" ],
    [ "2043", "04", "09", "18:57:49", "87", "T", "61.3N", "152.0E" ],
    [ "2043", "10", "03", "03:01:49", "88", "A", "61.0S", "35.3E" ],
    [ "2044", "02", "28", "20:24:39", "88", "A", "62.2S", "25.6W" ],
    [ "2044", "08", "23", "01:17:02", "88", "T", "64.3N", "120.4W" ],
    [ "2045", "02", "16", "23:56:07", "89", "A", "28.3S", "166.2W" ],
    [ "2045", "08", "12", "17:42:39", "89", "T", "25.9N", "78.5W" ],
    [ "2046", "02", "05", "23:06:26", "90", "A", "4.8N", "171.4W" ],
    [ "2046", "08", "02", "10:21:13", "90", "T", "12.7S", "15.2E" ],
    [ "2047", "01", "26", "01:33:18", "90", "P", "62.9N", "111.7E" ],
    [ "2047", "06", "23", "10:52:31", "91", "P", "65.8N", "178.0W" ],
    [ "2047", "07", "22", "22:36:17", "91", "P", "63.4S", "160.2E" ],
    [ "2047", "12", "16", "23:50:12", "91", "P", "66.4S", "6.6W" ],
    [ "2048", "06", "11", "12:58:53", "92", "A", "63.7N", "11.5W" ],
    [ "2048", "12", "05", "15:35:27", "92", "T", "46.1S", "56.4W" ],
    [ "2049", "05", "31", "13:59:59", "92", "A", "15.3N", "29.9W" ],
    [ "2049", "11", "25", "05:33:48", "93", "H", "3.8S", "95.2E" ],
    [ "2050", "05", "20", "20:42:50", "94", "H", "40.1S", "123.7W" ],
    [ "2050", "11", "14", "13:30:53", "95", "P", "69.5N", "1.0E" ],
    [ "2051", "04", "11", "02:10:39", "95", "P", "71.6N", "32.2E" ],
    [ "2051", "10", "04", "21:02:14", "96", "P", "72.0S", "117.7E" ],
    [ "2052", "03", "30", "18:31:53", "97", "T", "22.4N", "102.5W" ],
    [ "2052", "09", "22", "23:39:10", "98", "A", "25.7S", "175.0E" ],
    [ "2053", "03", "20", "07:08:19", "99", "A", "23.0S", "83.0E" ],
    [ "2053", "09", "12", "09:34:09", "100", "T", "21.5N", "41.7E" ],
    [ "2054", "03", "09", "12:33:40", "101", "P", "72.0S", "97.9E" ],
    [ "2054", "08", "03", "18:04:02", "102", "P", "69.8S", "121.3W" ],
    [ "2054", "09", "02", "01:09:34", "102", "P", "71.7N", "82.3W" ],
    [ "2055", "01", "27", "17:54:05", "103", "P", "69.5N", "112.2W" ],
    [ "2055", "07", "24", "09:57:50", "104", "T", "33.3S", "25.8E" ],
    [ "2056", "01", "16", "22:16:45", "105", "A", "3.9N", "153.5W" ],
    [ "2056", "07", "12", "20:21:59", "106", "A", "19.4N", "123.7W" ],
    [ "2057", "01", "05", "09:47:52", "107", "T", "39.2S", "35.2E" ],
    [ "2057", "07", "01", "23:40:15", "108", "A", "71.5N", "176.2W" ],
    [ "2057", "12", "26", "01:14:35", "109", "T", "84.9S", "21.8E" ],
    [ "2058", "05", "22", "10:39:25", "110", "P", "63.5S", "61.1E" ],
    [ "2058", "06", "21", "00:19:35", "110", "P", "65.9N", "9.9E" ],
    [ "2058", "11", "16", "03:23:07", "111", "P", "62.9N", "174.2E" ],
    [ "2059", "05", "11", "19:22:16", "112", "T", "10.7S", "100.4W" ],
    [ "2059", "11", "05", "09:18:15", "113", "A", "8.7N", "47.1E" ],
    [ "2060", "04", "30", "10:10:00", "114", "T", "28.0N", "20.9E" ],
    [ "2060", "10", "24", "09:24:10", "115", "A", "25.8S", "28.1E" ],
    [ "2061", "04", "20", "02:56:49", "116", "T", "64.5N", "59.2E" ],
    [ "2061", "10", "13", "10:32:10", "117", "A", "62.1S", "54.4W" ],
    [ "2062", "03", "11", "04:26:16", "118", "P", "61.0S", "147.1W" ],
    [ "2062", "09", "03", "08:54:27", "119", "P", "61.3N", "150.3E" ],
    [ "2063", "02", "28", "07:43:30", "120", "A", "25.2S", "77.7E" ],
    [ "2063", "08", "24", "01:22:11", "121", "T", "25.6N", "168.4E" ],
    [ "2064", "02", "17", "07:00:23", "122", "A", "7.0N", "69.7E" ],
    [ "2064", "08", "12", "17:46:06", "123", "T", "10.9S", "96.0W" ],
    [ "2065", "02", "05", "09:52:26", "124", "P", "62.2N", "21.9W" ],
    [ "2065", "07", "03", "17:33:52", "125", "P", "64.8N", "71.9E" ],
    [ "2065", "08", "02", "05:34:17", "125", "P", "62.7S", "46.5E" ],
    [ "2065", "12", "27", "08:39:56", "126", "P", "65.4S", "149.2W" ],
    [ "2066", "06", "22", "19:25:48", "127", "A", "70.1N", "96.4W" ],
    [ "2066", "12", "17", "00:23:40", "128", "T", "47.4S", "175.8E" ],
    [ "2067", "06", "11", "20:42:26", "129", "A", "21.0N", "130.2W" ],
    [ "2067", "12", "06", "14:03:43", "130", "H", "6.0S", "32.4W" ],
    [ "2068", "05", "31", "03:56:39", "131", "T", "31.0S", "123.2E" ],
    [ "2068", "11", "24", "21:32:30", "132", "P", "68.5N", "131.1W" ],
    [ "2069", "04", "21", "10:11:09", "133", "P", "71.0N", "101.3W" ],
    [ "2069", "05", "20", "17:53:18", "133", "P", "68.8S", "69.9W" ],
    [ "2069", "10", "15", "04:19:56", "134", "P", "71.6S", "5.5W" ],
    [ "2070", "04", "11", "02:36:09", "135", "T", "29.1N", "135.1E" ],
    [ "2070", "10", "04", "07:08:57", "136", "A", "32.8S", "60.4E" ],
    [ "2071", "03", "31", "15:01:06", "138", "A", "16.7S", "37.0W" ],
    [ "2071", "09", "23", "17:20:28", "139", "T", "14.2N", "76.7W" ],
    [ "2072", "03", "19", "20:10:31", "140", "P", "72.2S", "30.4W" ],
    [ "2072", "09", "12", "08:59:20", "141", "T", "69.8N", "102.0E" ],
    [ "2073", "02", "07", "01:55:59", "142", "P", "70.5N", "114.9E" ],
    [ "2073", "08", "03", "17:15:23", "143", "T", "43.2S", "89.4W" ],
    [ "2074", "01", "27", "06:44:15", "144", "A", "6.6N", "78.8E" ],
    [ "2074", "07", "24", "03:10:32", "145", "A", "12.8N", "133.7E" ],
    [ "2075", "01", "16", "18:36:04", "146", "T", "37.2S", "94.1W" ],
    [ "2075", "07", "13", "06:05:44", "147", "A", "63.1N", "95.2E" ],
    [ "2076", "01", "06", "10:07:27", "148", "T", "87.2S", "173.7W" ],
    [ "2076", "06", "01", "17:31:22", "149", "P", "64.4S", "51.2W" ],
    [ "2076", "07", "01", "06:50:43", "149", "P", "67.0N", "98.1W" ],
    [ "2076", "11", "26", "11:43:01", "150", "P", "63.7N", "40.1E" ],
    [ "2077", "05", "22", "02:46:05", "151", "T", "13.1S", "148.3E" ],
    [ "2077", "11", "15", "17:07:56", "152", "A", "7.8N", "70.8W" ],
    [ "2078", "05", "11", "17:56:55", "153", "T", "28.1N", "93.7W" ],
    [ "2078", "11", "04", "16:55:44", "154", "A", "27.8S", "83.3W" ],
    [ "2079", "05", "01", "10:50:13", "155", "T", "66.2N", "46.3W" ],
    [ "2079", "10", "24", "18:11:21", "156", "A", "63.4S", "160.6W" ],
    [ "2080", "03", "21", "12:20:15", "157", "P", "60.9S", "85.9E" ],
    [ "2080", "09", "13", "16:38:09", "158", "P", "61.1N", "25.8E" ],
    [ "2081", "03", "10", "15:23:31", "159", "A", "22.4S", "36.7W" ],
    [ "2081", "09", "03", "09:07:31", "160", "T", "24.6N", "53.6E" ],
    [ "2082", "02", "27", "14:47:00", "162", "A", "9.4N", "47.1W" ],
    [ "2082", "08", "24", "01:16:21", "163", "T", "10.3S", "151.8E" ],
    [ "2083", "02", "16", "18:06:36", "164", "P", "61.6N", "154.1W" ],
    [ "2083", "07", "15", "00:14:23", "165", "P", "64.0N", "37.7W" ],
    [ "2083", "08", "13", "12:34:41", "165", "P", "62.1S", "67.5W" ],
    [ "2084", "01", "07", "17:30:24", "166", "P", "64.4S", "68.5E" ],
    [ "2084", "07", "03", "01:50:26", "167", "A", "75.0N", "169.1W" ],
    [ "2084", "12", "27", "09:13:48", "168", "T", "47.3S", "47.7E" ],
    [ "2085", "06", "22", "03:21:16", "169", "A", "26.2N", "131.3E" ],
    [ "2085", "12", "16", "22:37:48", "170", "A", "7.3S", "160.8W" ],
    [ "2086", "06", "11", "11:07:14", "171", "T", "23.2S", "12.5E" ],
    [ "2086", "12", "06", "05:38:55", "172", "P", "67.4N", "96.2E" ],
    [ "2087", "05", "02", "18:04:42", "173", "P", "70.3N", "127.6E" ],
    [ "2087", "06", "01", "01:27:14", "173", "P", "67.8S", "165.4E" ],
    [ "2087", "10", "26", "11:46:57", "174", "P", "71.0S", "130.5W" ],
    [ "2088", "04", "21", "10:31:49", "175", "T", "36.0N", "15.1E" ],
    [ "2088", "10", "14", "14:48:05", "177", "A", "39.7S", "56.0W" ],
    [ "2089", "04", "10", "22:44:42", "178", "A", "10.2S", "154.8W" ],
    [ "2089", "10", "04", "01:15:23", "179", "T", "7.4N", "162.8E" ],
    [ "2090", "03", "31", "03:38:08", "180", "P", "72.1S", "156.3W" ],
    [ "2090", "09", "23", "16:56:36", "181", "T", "60.7N", "40.5W" ],
    [ "2091", "02", "18", "09:54:40", "182", "P", "71.2N", "17.8W" ],
    [ "2091", "08", "15", "00:34:43", "183", "T", "55.6S", "150.5E" ],
    [ "2092", "02", "07", "15:10:20", "184", "A", "9.9N", "48.7W" ],
    [ "2092", "08", "03", "09:59:33", "185", "A", "5.6N", "30.3E" ],
    [ "2093", "01", "27", "03:22:16", "186", "T", "34.1S", "136.4E" ],
    [ "2093", "07", "23", "12:32:04", "187", "A", "54.6N", "1.3E" ],
    [ "2094", "01", "16", "18:59:03", "189", "T", "84.8S", "10.6W" ],
    [ "2094", "06", "13", "00:22:11", "190", "P", "65.3S", "163.6W" ],
    [ "2094", "07", "12", "13:24:35", "190", "P", "68.0N", "152.8E" ],
    [ "2094", "12", "07", "20:05:56", "191", "P", "64.7N", "95.0W" ],
    [ "2095", "06", "02", "10:07:40", "192", "T", "16.7S", "37.2E" ],
    [ "2095", "11", "27", "01:02:57", "193", "A", "7.2N", "169.8E" ],
    [ "2096", "05", "22", "01:37:14", "194", "T", "27.3N", "153.4E" ],
    [ "2096", "11", "15", "00:36:15", "195", "A", "29.7S", "163.3E" ],
    [ "2097", "05", "11", "18:34:31", "196", "T", "67.4N", "149.5W" ],
    [ "2097", "11", "04", "02:01:25", "197", "A", "65.8S", "86.8E" ],
    [ "2098", "04", "01", "20:02:31", "198", "P", "61.0S", "38.1W" ],
    [ "2098", "09", "25", "00:31:16", "199", "P", "61.1N", "101.0W" ],
    [ "2098", "10", "24", "10:36:11", "200", "P", "61.8S", "95.5W" ],
    [ "2099", "03", "21", "22:54:32", "201", "A", "20.0S", "149.0W" ],
    [ "2099", "09", "14", "16:57:53", "202", "T", "23.4N", "62.8W" ] ]